struct InstanceInput {
	@location(2) position: vec3<f32>,
	@location(3) size: f32,
	@location(4) expires: f32,
	@location(5) texture_index: u32,
};

struct VertexInput {
	@location(0) position: vec3<f32>,
	@location(1) texture_coords: vec2<f32>,
}

struct VertexOutput {
	@builtin(position) position: vec4<f32>,
	@location(0) texture_coords: vec2<f32>,
	@location(1) texture_index: u32,
}


@group(1) @binding(0)
var<uniform> global: GlobalUniform;
struct GlobalUniform {
	camera_position: vec2<f32>,
	camera_zoom: vec2<f32>,
	camera_zoom_limits: vec2<f32>,
	window_size: vec2<f32>,
	window_aspect: vec2<f32>,
	starfield_texture: vec2<u32>,
	starfield_tile_size: vec2<f32>,
	starfield_size_limits: vec2<f32>,
	current_time: vec2<f32>,
};


@group(0) @binding(0)
var texture_array: binding_array<texture_2d<f32>>;
@group(0) @binding(1)
var sampler_array: binding_array<sampler>;




@vertex
fn vertex_main(
	vertex: VertexInput,
	instance: InstanceInput,
) -> VertexOutput {

	var out: VertexOutput;
	out.texture_coords = vertex.texture_coords;
	out.texture_index = instance.texture_index;

	if instance.expires < global.current_time.x {
		out.position = vec4<f32>(2.0, 2.0, 0.0, 1.0);
		return out;
	}

	var scale: f32 = instance.size / global.camera_zoom.x;
	var pos: vec2<f32> = vec2(vertex.position.x, vertex.position.y);

	pos = pos * vec2<f32>(
		1.0 * scale / global.window_aspect.x,
		scale
	);

	var ipos: vec2<f32> = vec2(instance.position.x, instance.position.y) - global.camera_position;
	pos = pos + vec2<f32>(
		ipos.x / (global.camera_zoom.x/2.0) / global.window_aspect.x,
		ipos.y / (global.camera_zoom.x/2.0)
	);

	out.position = vec4<f32>(pos, 0.0, 1.0) * instance.position.z;
	return out;
}


@fragment
fn fragment_main(in: VertexOutput) -> @location(0) vec4<f32> {
	return textureSampleLevel(
		texture_array[in.texture_index],
		sampler_array[0],
		in.texture_coords,
		0.0
	).rgba;
}