use cgmath::{Deg, InnerSpace, Point3, Vector2};
use rapier2d::{
	dynamics::{RigidBody, RigidBodyHandle},
	geometry::ColliderHandle,
};

use crate::util;
use galactica_gameobject as object;
use galactica_render::ObjectSprite;

/// A single projectile in the world
#[derive(Debug)]
pub struct ProjectileWorldObject {
	/// TODO
	pub projectile: object::Projectile,

	/// TODO
	pub rigid_body: RigidBodyHandle,

	/// TODO
	pub collider: ColliderHandle,
}

impl ProjectileWorldObject {
	/// Make a new projectile
	pub fn new(
		projectile: object::Projectile,
		rigid_body: RigidBodyHandle,
		collider: ColliderHandle,
	) -> Self {
		ProjectileWorldObject {
			rigid_body,
			collider,
			projectile,
		}
	}

	/// Get this projectiles' sprite
	pub fn get_sprite(&self, r: &RigidBody) -> ObjectSprite {
		let pos = util::rigidbody_position(r);
		let rot = util::rigidbody_rotation(r);

		// Sprites point north at 0 degrees
		let ang: Deg<f32> = rot.angle(Vector2 { x: 1.0, y: 0.0 }).into();

		ObjectSprite {
			texture: self.projectile.content.sprite_texture,
			pos: Point3 {
				x: pos.x,
				y: pos.y,
				z: 1.0,
			},
			size: self.projectile.content.size,
			angle: -ang,
			children: None,
		}
	}
}
